## $Id: test_browser.py 12342 2014-12-30 10:19:47Z henrik $
## 
## Copyright (C) 2014 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""
Test the products-related UI components.
"""
import shutil
import tempfile
import pytz
import base64
from decimal import Decimal
from datetime import datetime, timedelta, date
from StringIO import StringIO
import os
import grok
from zc.async.testing import wait_for_result
from zope.event import notify
from zope.component import createObject, queryUtility, getUtility
from zope.component.hooks import setSite, clearSite
from zope.catalog.interfaces import ICatalog
from zope.security.interfaces import Unauthorized
from zope.securitypolicy.interfaces import IPrincipalRoleManager
from zope.testbrowser.testing import Browser
from hurry.workflow.interfaces import IWorkflowInfo, IWorkflowState
from waeup.ikoba.testing import FunctionalLayer, FunctionalTestCase
from waeup.ikoba.app import Company
from waeup.ikoba.interfaces import (
    IUserAccount, IJobManager, APPROVED, NotIdValue)
from waeup.ikoba.authentication import LocalRoleSetEvent
from waeup.ikoba.tests.test_async import FunctionalAsyncTestCase
from waeup.ikoba.browser.tests.test_pdf import samples_dir
from waeup.ikoba.products.productoptions import ProductOption

def lookup_submit_value(name, value, browser):
    """Find a button with a certain value."""
    for num in range(0, 100):
        try:
            button = browser.getControl(name=name, index=num)
            if button.value.endswith(value):
                return button
        except IndexError:
            break
    return None

class ProductsFullSetup(FunctionalTestCase):
    # A test case that only contains a setup and teardown
    #
    # Complete setup for products handlings is rather complex and
    # requires lots of things created before we can start. This is a
    # setup that does all this, creates a company,
    # etc.  so that we do not have to bother with that in different
    # test cases.

    layer = FunctionalLayer

    def setUp(self):
        super(ProductsFullSetup, self).setUp()

        # Setup a sample site for each test
        app = Company()
        self.dc_root = tempfile.mkdtemp()
        app['datacenter'].setStoragePath(self.dc_root)

        # Prepopulate the ZODB...
        self.getRootFolder()['app'] = app
        # we add the site immediately after creation to the
        # ZODB. Catalogs and other local utilities are not setup
        # before that step.
        self.app = self.getRootFolder()['app']
        # Set site here. Some of the following setup code might need
        # to access grok.getSite() and should get our new app then
        setSite(app)

        self.logfile = os.path.join(
            self.app['datacenter'].storage, 'logs', 'main.log')

        # Add user
        self.app['users'].addUser(name='john', password='johnpwd',
                                  title='John Tester',
                                  email='john@tester.de')

        # Add some product
        self.product = createObject('waeup.Product')
        self.product.product_id = u'LIC'
        self.assertRaises(
            NotIdValue, setattr, self.product, 'product_id', u'id with spaces')
        self.product.title = u'Our License'
        self.app['products'].addProduct(self.product)

        # Add session configuration
        self.app['configuration'].carry_over = True
        self.login_path = 'http://localhost/app/login'
        self.container_path = 'http://localhost/app/products'
        self.manage_container_path = self.container_path + '/manage'
        self.add_product_path = self.container_path + '/addproduct'
        self.product_path = self.container_path + '/LIC'
        self.manage_product_path = self.container_path + '/LIC/manage'

        # Put the prepopulated site into test ZODB and prepare test
        # browser
        self.browser = Browser()
        self.browser.handleErrors = False

    def tearDown(self):
        super(ProductsFullSetup, self).tearDown()
        clearSite()
        shutil.rmtree(self.dc_root)

class ProductsContainerUITests(ProductsFullSetup):
    # Tests for ProductsContainer class views and pages

    layer = FunctionalLayer

    def test_anonymous_access(self):
        # Anonymous users can view products containers ...
        self.browser.open(self.container_path)
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        # ... but not manage
        self.assertRaises(
            Unauthorized, self.browser.open, self.manage_container_path)
        return

    def test_manage_access(self):
        # Managers can access the view page of products
        # containers and can perform actions
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        self.browser.open(self.container_path)
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        self.assertEqual(self.browser.url, self.container_path)
        self.browser.getLink("Manage").click()
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        self.assertEqual(self.browser.url, self.manage_container_path)
        return

    def test_add_delete_products(self):
        # Managers can add search and remove products
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        self.browser.open(self.manage_container_path)
        self.browser.getControl("Add product").click()
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        self.assertEqual(self.browser.url, self.add_product_path)
        self.browser.getControl(name="form.product_id").value = 'bob'
        self.browser.getControl(name="form.title").value = 'My product'
        self.browser.getControl("Create product").click()
        self.assertTrue('Product bob added' in self.browser.contents)
        self.assertEqual(len(self.app['products'].keys()), 2)
        self.assertEqual(self.app['products']['bob'].title, 'My product')
        ctrl = self.browser.getControl(name='val_id')
        ctrl.getControl(value='bob').selected = True
        self.browser.getControl("Remove selected", index=0).click()
        self.assertTrue('Successfully removed' in self.browser.contents)
        self.assertEqual(len(self.app['products'].keys()), 1)


class ProductsUITests(ProductsFullSetup):
    # Tests for Products class views and pages

    layer = FunctionalLayer

    def test_anonymous_access(self):
        # Anonymous users can access products ...
        self.browser.open(self.product_path)
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        # ... but not manage products
        self.assertRaises(
            Unauthorized, self.browser.open, self.manage_product_path)
        return

    def test_manage_access(self):
        # Managers can access the view pages of products
        # and can perform actions
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        self.browser.open(self.product_path)
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        # There is a Manage button
        self.browser.getLink("Manage").click()
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        self.assertEqual(self.browser.url, self.manage_product_path)
        self.browser.getControl(name="form.title").value = 'New license'
        self.browser.getControl("Save").click()
        self.assertEqual(self.app['products']['LIC'].title, 'New license')
        # Managers can asign local roles
        self.browser.getControl(name="user").value = ['john']
        self.browser.getControl(
            name="local_role").value = ['waeup.local.ProductManager']
        self.browser.getControl("Add local role").click()
        self.assertTrue('john|waeup.local.ProductManager' in self.browser.contents)
        self.assertEqual(
            self.app['users']['john'].getLocalRoles().get(
                'waeup.local.ProductManager')[0],
            self.product)
        # There is a View button wich directs to the index page
        self.browser.getLink("View").click()
        self.assertEqual(self.browser.url, self.product_path + '/index')
        # If products is deleted also the local role disappears
        self.browser.open(self.manage_container_path)
        ctrl = self.browser.getControl(name='val_id')
        ctrl.getControl(value='LIC').selected = True
        self.browser.getControl("Remove selected", index=0).click()
        self.assertEqual(
            self.app['users']['john'].getLocalRoles().get('waeup.local.ProductManager'),
            None)
        return

    def test_product_options(self):
        prodoptions = ProductOption()
        prodoptions.title = u'My option'
        prodoptions.fee = Decimal('99.9')
        prodoptions.currency = 'USD'
        self.product.options = [prodoptions,]
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        self.browser.open(self.product_path)
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        self.assertEqual(self.product.options[0], prodoptions)
        self.browser.open(self.manage_product_path)
        self.browser.getControl("Save").click()
        # applyData must not exchange the object
        self.assertEqual(self.product.options[0], prodoptions)
        # and thus not add a logging message (the log file
        # do not even exist so far)
        self.assertRaises(IOError, open, self.logfile, 'read')
        # Now we change the content after opening the manage page
        self.browser.open(self.manage_product_path)
        prodoptions.title = u'My new option'
        self.assertEqual(self.product.options[0].title, 'My new option')
        # but save the opened form with the old data.
        self.browser.getControl("Save").click()
        # The object has been replaced,
        self.assertTrue(self.product.options[0] != prodoptions)
        # the old title has been restored
        self.assertEqual(self.product.options[0].title, 'My option')
        # and a logging message has been added.
        logcontent = open(self.logfile).read()
        self.assertTrue('saved: options' in logcontent)
